%% ------------------------------------------------------------------------
% batch files for DSGE model
% ------------------------------------------------------------------------
clear all;
clc;
diary off;

% ------------------------------------------------------------------------
%  Add path to use functions for the DSGE estimation
% ------------------------------------------------------------------------
%  Toolbox folder includes
%   1. CSOPT     : Sims' optimization toolbox
%   2. LRE       : Sims' linear rational expectation model solver
%   3. DSGE_SOLVE: solution algorithm for dsge model
%   4. DSGE_SEL  : Other utility functoins (log pdf, prior evaluation, etc)
%   5. OTHEROPT  : CMAES optimization
% ------------------------------------------------------------------------
workpath          = pwd;
path_original     = path;
path_DSGE_TOOLBOX = genpath([workpath, filesep, 'DSGE_TOOLBOX']);
addpath(path_DSGE_TOOLBOX);

% estimation settings
dsge_spec;

% set path for this application
pathspec;


%% ------------------------------------------------------------------------
% Step 1: Generate draws from the posterior based on Rsample
% - obtain posterior max
% - hessian at pmod
% ------------------------------------------------------------------------
% load dataset
% load filename.mat
cd(datapath);
load BC_SMOD_filename.mat
cd(workpath);
filename_est  = filenamestr_est;
filename_pred = filenamestr_rf;

% choose dataset str
sampleind = estinfo.sampleind;
datasetstr = filename_est{sampleind,1};
predsetstr = filename_pred{sampleind,1};

% load data
dsge_loaddata;
estinfo.sbdate = (tiR>=1985)*[1,1,1]; %1985 is break date as in Primiceri

%% posterior maximization
YY    = YYR(Tbegin:Tend,:);
opath = Rpostpath; %output path

estinfo.specname = specname;

if estinfo.sbsv ~= 1
    dsge_pm;
elseif estinfo.sbsv == 1
    dsge_pm_sbsv;
end


%% compute hessian
YY    = YYR(Tbegin:Tend,:);
lpath = Rpostpath; %load path
opath = Rpostpath; %output path

useHess = 2; % 1 if we construct prop.dist. using hess (original code)
             % 2 if we compute hessian based on Romberg approximation (it seems to be more accurate and robust)
             % 0 if we construct prop.dist. using pre-run MH algorithm

estinfo.specname = specname;
if useHess == 1
    dsge_hess;
elseif useHess == 2
    if estinfo.sbsv ~= 1
        dsge_hess_adaptive;
    elseif estinfo.sbsv == 1
        dsge_hess_sbsv_adaptive;
    end
end

%% MCMC sampler - MH
YY     = YYR(Tbegin:Tend,:);

cc0     = 1; % scale for initialization (scale for MH draws is specficied in dsge_spec)
useHess = 2; % 1 if we construct prop.dist. using hess (original code)
             % 2 if we compute hessian based on Romberg approximation (it seems to be more accurate and robust)
             % 0 if we construct prop.dist. using pre-run MH algorithm

lpath  = Rpostpath;
opath  = Rparapath;

samplepath = [specpath, '\sample', num2str(sampleind)];
opath  = samplepath;

mhrun      = estinfo.mhrun;
nsimul     = nsimul_;
% nparticle  = estinfo.nparticle;
% n_particle = estinfo.nparticle;
ncounter   = ncounter_;
saveSV = estinfo.SVsave;

if estinfo.sbsv ~=1
    if estinfo.SVdo == 1
        if estinfo.Qone == 1
            dsge_mh_sv_adaptive_Qone;
        elseif estinfo.Qone ==0
            dsge_mh_sv_adaptive;
        end
    elseif estinfo.SVdo == 0
        dsge_mh_adaptive;
    end
elseif estinfo.sbsv == 1
    dsge_mh_sbsv_adaptive;
end

%% Generate Prediction
nburn  = nburn_;
nuse   = nuse_;
npaths = npaths_;

lpath  = samplepath;
opath  = samplepath;

YYplus = YYP;
tiplus = tiP;
YY     = YYR(Tbegin:Tend, :);

if estinfo.sbsv == 0
    if estinfo.SVdo == 0
        dsge_pred;
    elseif estinfo.SVdo== 1
        
        if estinfo.Qone == 0
            dsge_pred_sv;
        elseif estinfo.Qone == 1
            dsge_pred_sv_Qone;
        end
    end
elseif estinfo.sbsv == 1
    dsge_pred_sbsv;
end



